package rest.resources;

import com.google.gson.Gson;
import java.net.URI;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.inject.Inject;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.FormParam;
import javax.ws.rs.Produces;
import javax.ws.rs.GET;
import javax.ws.rs.OPTIONS;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PUT;
import javax.ws.rs.PathParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.ResponseBuilder;
import rest.model.Person;
import rest.model.PersonList;

/**
 * REST Web Service
 *
 * @author hajo
 */
@Path("list")
public class PersonResource {

    private static final Logger LOG = Logger.getAnonymousLogger();
    private static final Level I = Level.INFO;

    @Context
    private UriInfo uriInfo;
   
    @Inject
    private PersonList list;
    // Non standard JSON API
    private final Gson gson = new Gson();

    // Provides access to application and request URI information. 
    // Relative URIs are relative to the base URI of the application
    @GET
    @Path("{id: \\d+}")
    @Produces({MediaType.APPLICATION_JSON + ";charset=utf-8"})
    public Response find(@PathParam("id") Long id) {
        Person p = list.getById(id);
        if (p != null) {
            return addCORSheaders(Response.ok(gson.toJson(p))).build();
        } else {
            return addCORSheaders(Response.status(Response.Status.NOT_FOUND)).build();
        }
    }

    @GET
    @Produces({MediaType.APPLICATION_JSON + ";charset=utf-8"})
    public Response findAll() {
        List<Person> persons = list.getPersons();
        return addCORSheaders(Response.ok(gson.toJson(persons))).build();

    }

    @GET
    @Produces({MediaType.APPLICATION_JSON + ";charset=utf-8"})
    @Path("count")
    public Response count() {
        int count = list.size();
        JSONPrimitive<Integer>  p = new JSONPrimitive("count", count);
        return addCORSheaders(Response.ok(gson.toJson(p))).build();
    }
    
    class JSONPrimitive<T> {
        public String key;
        T value;

        public JSONPrimitive(String key, T value) {
            this.key = key;
            this.value = value;
        }
        
    };


    @POST
    @Consumes({MediaType.APPLICATION_FORM_URLENCODED})
    public Response add(@FormParam("name") String name) {
        Person person = new Person(null, name);
        list.add(person);
        URI noteUri = uriInfo
                .getAbsolutePathBuilder()
                .path(String.valueOf(person.getId()))
                .build(person);
        // Set it to 201 (created) and setting response header 'Location'
        // Inspect with cURL
        return addCORSheaders(Response.created(noteUri)).build();
    }

    @DELETE
    @Path("{id: \\d+}")
    public Response delete(@PathParam("id") Long id) {
        // A successful response SHOULD be 200 (OK) if the response 
        // includes an entity describing the status, 202 (Accepted) if 
        // the action has not yet been enacted, or 204 (No Content) if 
        // the action has been enacted but the response does not 
        // include an entity.
        list.delete(id);
        return addCORSheaders(Response.noContent()).build();
    }

    @PUT
    @Path("{id: \\d+}")
    @Consumes({MediaType.APPLICATION_FORM_URLENCODED})
    public Response update(@PathParam("id") Long id,
            @FormParam("name") String name, @FormParam("done") boolean done) {
        Person person = new Person(id, name);
        list.update(person);

        //If an existing resource is modified, either the 200 (OK) or 204 
        //(No Content) response codes > SHOULD be sent to indicate 
        //successful completion of the request.
        return addCORSheaders(Response.ok()).build();
    }

    @OPTIONS
    //@Path("{*}")
    @Path("/{path: .*}")
    public Response options() {
        LOG.log(Level.INFO, "OPTIONS");
        return addCORSheaders(Response.ok()).build();
    }

    // Used by CORS
    // ContainerFilters doens't work in TomEE (!?!?), do it by hand ... 
    // Possibly GlassFish can handle?
    // Not needed if backend and client bundled (i.e. same origin)
    private ResponseBuilder addCORSheaders(ResponseBuilder builder) {
        return builder.header("Access-Control-Allow-Origin", "*")
                .header("Access-Control-Allow-Methods", "GET, POST, PATCH, OPTION, DELETE, PUT")
                .header("Access-Control-Allow-Headers", "X-Requested-With, Content-Type");
    }

}
